;Digital Transmission of Analog Variable Trough Long Distance I2C Bus
;The data is acquired by the MCP3221 ADC and generated by the MCP4726 DAC
;The I2C signals are buffered by the PCA9605 driver
;Camilo Quintns, August 2014.

LIST P=PIC18F45K20
#INCLUDE <P18F45K20.INC>

CONFIG FOSC=INTIO67 ;Internal oscillator clock
CBLOCK 0x00		    ;Data memory
	i2c_dato
	i2c_dato1_rx
	i2c_dato2_rx		
	DelayCtr2
	DelayCtr1
ENDC

ORG 0x00
	GOTO main
ORG 0x20

;subrutina de inicializacin del i2c
i2c_ini
	BSF	SSPSTAT,SMP		; I2C slew rate control disabled
	BSF	SSPCON1,SSPM3	; I2C master mode in hardware
	BSF	SSPCON1,SSPEN	; enable SSP module
	;MOVLW	0x01	;(ADD+1)*4]/Fosc=SCL_period  => 83.3 kHz con Fosc=1 MHz
	MOVLW	0x20	;(ADD+1)*4]/Fosc=SCL_period  => 120kHz con Fosc=16 MHz
	;MOVLW	0x27	;(ADD+1)*4]/Fosc=SCL_period  => 100kHz con Fosc=16 MHz
	;MOVLW	0x10	;(ADD+1)*4]/Fosc=SCL_period	 => 400kHz con Fosc=16 MHz
	;MOVLW	0x13	;(ADD+1)*4]/Fosc=SCL_period	 => 200kHz con Fosc=16 MHz
	MOVWF	SSPADD	;I2C clock 

	BSF	TRISC,3		; I2C SCL pin is input
	BSF	PORTC,3		; (will be controlled by SSP)
	BSF	TRISC,4		; I2C SDA pin is input
	BSF	PORTC,4		; (will be controlled by SSP)
RETURN
;fin de subrutina

;subrutina de inicio de transmisin i2c
;envio del bit de START
i2c_start
	BSF		SSPCON2,SEN	; Send start bit
bit_start_borrado
	BTFSC	SSPCON2,SEN	; Has SEN cleared yet?
	BRA	bit_start_borrado
RETURN
;fin de subrutina

;subrutina que envia un STOP
i2c_stop
	BSF		SSPCON2,PEN	; send stop bit
	BTFSC	SSPCON2,PEN	; has stop bit been sent?
	BRA		$-2		; no, loop back to test
RETURN
;fin de subrutina

;subrutina de transmisin i2c
;envio del dato desde el WREG al dipositivo
i2c_tx
	BCF		PIR1,SSPIF	; clear interrupt flag

	MOVFF	i2c_dato,SSPBUF	; move device data to SSPBUF
	BTFSS	PIR1,SSPIF	; has SSP completed sending device Address?
	BRA		$-2			; no, loop back to test
	BTFSC	SSPCON2,ACKSTAT	; has slave sent ACK?
	BRA		$-2			; no, try again
RETURN
;fin de subrutina

i2c_rx_ack
	BCF		PIR1,SSPIF		; clear interrupt flag
	BSF		SSPCON2,RCEN	; enable receive mode
	BTFSS	PIR1,SSPIF		; has SSP received a data byte?
	BRA		$-2		; no, loop back to test
	MOVFF	SSPBUF,i2c_dato1_rx
	BCF		SSPCON2,ACKDT	; si ACK
	BSF		SSPCON2,ACKEN	; send ACKDT bit
	BTFSC	SSPCON2,ACKEN	; has ACKDT bit been sent yet?
	BRA		$-2		; no, loop back to test
RETURN
;fin de subrutina

i2c_rx_nack
	BCF		PIR1,SSPIF		; clear interrupt flag
	BSF		SSPCON2,RCEN	; enable receive mode
	BTFSS	PIR1,SSPIF		; has SSP received a data byte?
	BRA		$-2		; no, loop back to test
	MOVFF	SSPBUF,i2c_dato2_rx
	BSF		SSPCON2,ACKDT	; no ACK
	BSF		SSPCON2,ACKEN	; send ACKDT bit
	BTFSC	SSPCON2,ACKEN	; has ACKDT bit been sent yet?
	BRA		$-2		; no, loop back to test
	BCF		SSPCON2,RCEN	; disable receive mode
RETURN
;fin de subrutina

;rutina de retardo
delay
	movlw	0x20
	movwf	DelayCtr2	; preset
	clrf	DelayCtr1	; clear counter
Delay1
	decfsz	DelayCtr1	; decrement counter
	bra		Delay1		; back to top of loop
	decfsz	DelayCtr2	; decrement counter
	bra		Delay1		; back to top of loop
	RETURN
;fin de subrutina

;subrutina que lee un dato del ADC MCP3221, la direccin es: 0x9A
;se reciben dos bytes, el primero tiene en sus 4 LSB los bits 11-8 del dato
;el segundo byte es la parte baja de la conversin, los bits  7-0 del dato
read_ADC
						;inicio transmisin y envio direccin esclavo
	MOVLW	0x9B		;se pone a uno el bit 0 para indicar un lectura
	MOVWF	i2c_dato    ;se pone en el registro de dato la direccin
	CALL	i2c_start   ;se genera una secuencia de inicio
	CALL	i2c_tx	    ;se transmite la direccin al esclavo (el ADC)
	CALL	i2c_rx_ack  ;recepcin del primer byte y reconocimiento
	CALL	i2c_rx_nack ;recepcin del segundo byte y no reconocimiento
	CALL 	i2c_stop	;se para la transmisin
	RETURN
;fin de subrutina

;subrutina que escribe un dato del DAC MCP4726, la direccin es: 0xC0
;se escriben dos bytes, el primero tiene en sus 4 LSB los bits 11-8 del dato
;el segundo byte es la parte baja de la conversin, los bits  7-0 del dato
write_DAC
						;inicio transmisin y envio direccin esclavo
	MOVLW	0xC0		;se pone a cero el bit 0 para indicar una escritura
	MOVWF	i2c_dato    ;se pone en el registro de dato la direccin
	CALL	i2c_start   ;se genera una secuencia de inicio
	CALL	i2c_tx		;se transmite la direccin al DAC
	MOVFF 	i2c_dato1_rx,i2c_dato
	
	;MOVLW	0x0A
	;MOVWF	i2c_dato

	CALL	i2c_tx 
	MOVFF 	i2c_dato2_rx,i2c_dato	
	
	;MOVLW	0xE2
	;MOVWF	i2c_dato

	CALL	i2c_tx
	CALL 	i2c_stop	;se para la transmisin
	RETURN
;fin de subrutina

write_DAC_add
						;inicio transmisin y envio direccin esclavo
	MOVLW	0xC0		;se pone a cero el bit 0 para indicar una escritura
	MOVWF	i2c_dato    ;se pone en el registro de dato la direccin
	CALL	i2c_start   ;se genera una secuencia de inicio
	CALL	i2c_tx		;se transmite la direccin al DAC
	CALL 	i2c_stop	;se para la transmisin
	RETURN
;fin de subrutina
main
	BSF		OSCCON,6	;sets the clock to 16 MHz
	CALL	i2c_ini
	CALL 	delay
loop
	CALL	read_ADC
	CALL	write_DAC
	;CALL 	delay
	BRA loop
END